import {Injectable} from '@angular/core';
import {BehaviorSubject, combineLatest, merge, Observable, of, Subject} from 'rxjs';
import {
  Activity,
  ApplicationContext,
  CarePlan,
  CarePlanService,
  CreateRegularObservationsRequest,
  CreateUltrasoundObservationsRequest,
  DataCard,
  DataCardResponse,
  DataCardService,
  ObservationService
} from 'facade-api';
import {map, shareReplay, switchMap, tap} from 'rxjs/operators';
import {UserContextService} from './user-context.service';
import {PatientContextService} from './patient-context.service';

/**
 * Shared data service
 */
@Injectable({
  providedIn: 'root'
})
export class DataSourceService {
  constructor(
    private observationsService: ObservationService,
    private carePlanService: CarePlanService,
    private dataCardService: DataCardService,
    private userContextService: UserContextService,
    private patientContextService: PatientContextService
  ) {
  }

  // activate data Loading
  private innerDataLoadingActivator: Subject<undefined> = new BehaviorSubject(undefined);

  private innerActiveDataCard: Subject<DataCardResponse | undefined> = new Subject<DataCardResponse | undefined>();

  // load data when innerDataLoadingActivator is emitting and when role is selected
  private dataLoadingActivator: Observable<undefined> = merge(
      this.innerDataLoadingActivator,
      this.userContextService.contextIsInitialized$
  ).pipe(map(_ => undefined));
  // FIXME: We dont need this... update careplan service and friends to not require the param;
  private applicationContext: ApplicationContext = {} as any;

  public activeDataCard$: Observable<DataCardResponse | undefined> = merge(
    this.patientContextService.episodeOfCare$.pipe(
      switchMap((id: string | undefined) => this.getActiveDataCard(id)),
      shareReplay()
    ),
    this.dataLoadingActivator.pipe(
        switchMap(() => this.dataCardService.maternityDataCardGetAll().pipe(
          map((cards: DataCardResponse[]) =>
            cards.find((x) => x.episodeOfCareIdentifier === this.patientContextService.getCurrentEpisodeOfCare()))
        )),
        shareReplay(),
    ),
    this.innerActiveDataCard
  );

  public allDataCards$: Observable<DataCardResponse[]> = this.dataLoadingActivator.pipe(
    switchMap(() => this.dataCardService.maternityDataCardGetAll()
      .pipe(
        tap((x?: DataCardResponse[]) => this.innerActiveDataCard.next(
          x?.find(card => card.episodeOfCareIdentifier === this.patientContextService.getCurrentEpisodeOfCare()))
        ),
        map((x?: DataCardResponse[]) => x ?? [])
      )
    ),
    shareReplay()
  );

  public carePlan$: Observable<CarePlan | undefined> = this.dataLoadingActivator.pipe(
    switchMap(() => {
      const episodeOfCareId = this.patientContextService.getCurrentEpisodeOfCare();
      if (episodeOfCareId) {
        return this.carePlanService.maternityCarePlanGet();
      }
      return of(undefined);
    }),
    shareReplay()
  );

  public activities$: Observable<Array<Activity>> = this.carePlan$.pipe(
    map((careplan: CarePlan | undefined) => careplan?.activities || [])
  );

  public getActivityById(id: string): Observable<Activity|null> {
    return this.carePlan$.pipe(
      map((careplan?: CarePlan) => careplan?.activities?.find(activity => activity.id === id) || null)
    );
  }

  public updateDataCard(
    documentUniqueId: string,
    version: string,
    req: DataCard
  ): Observable<DataCardResponse> {
    return this.dataCardService.maternityDataCardPut(documentUniqueId, version, req).pipe(
      tap((x: DataCardResponse) => {
        this.innerActiveDataCard.next(x);
      })
    );
  }

  public createUltrasoundObservations(observations: CreateUltrasoundObservationsRequest): Observable<void> {
    return this.observationsService.maternityUltrasoundObservationsPost(this.applicationContext, observations);
  }

  public createRegularObservations(observations: CreateRegularObservationsRequest): Observable<void> {
    return this.observationsService.maternityRegularObservationsPost(this.applicationContext, observations);
  }

  public refresh(): void {
    this.innerDataLoadingActivator.next();
  }

  private getActiveDataCard(episodeOfCareId?: string): Observable<DataCardResponse | undefined> {
    if (episodeOfCareId) {
      return this.allDataCards$.pipe(
        map((cards: DataCardResponse[]) => cards.find(x => x.episodeOfCareIdentifier === episodeOfCareId))
      );
    }
    return of(undefined);
  }
}
